@@ -4,6 +4,7 @@ worker_config_serialize <- function(config) {
     spark_config_value(config, "sparklyr.worker.gateway.port", "8880"),
     spark_config_value(config, "sparklyr.worker.gateway.address", "localhost"),
     if (isTRUE(config$profile)) "TRUE" else "FALSE",
+    if (isTRUE(config$schema)) "TRUE" else "FALSE",
     sep = ";"
   )
 }
@@ -15,6 +16,7 @@ worker_config_deserialize <- function(raw) {
     debug = as.logical(parts[[1]]),
     sparklyr.gateway.port = as.integer(parts[[2]]),
     sparklyr.gateway.address = parts[[3]],
-    profile = as.logical(parts[[4]])
+    profile = as.logical(parts[[4]]),
+    schema = as.logical(parts[[5]])
   )
 }
