@@ -33,7 +33,7 @@ spark_worker_main <- function(
     sc <- spark_worker_connect(sessionId, backendPort, config)
     worker_log("is connected")
 
-    spark_worker_apply(sc)
+    spark_worker_apply(sc, config)
 
     if (identical(config$profile, TRUE)) {
       # utils::Rprof(NULL)
