@@ -11,7 +11,37 @@ class Sources {
 #' @keywords internal
 #' @export
 spark_config_value <- function(config, name, default = NULL) {
-  if (!name %in% names(config)) default else config[[name]]
+  if (getOption("sparklyr.test.enforce.config", FALSE) && any(startsWith(name, "sparklyr."))) {
+    settings <- get("spark_config_settings")()
+    if (!any(name %in% settings$name)) {
+      stop("Config value '", name[[1]], "' not described in spark_config_settings()")
+    }
+  }
+
+  name_exists <- name %in% names(config)
+  if (!any(name_exists)) {
+    name_exists <- name %in% names(options())
+    if (!any(name_exists)) {
+      value <- default
+    } else {
+      name_primary <- name[name_exists][[1]]
+      value <- getOption(name_primary)
+    }
+  } else {
+    name_primary <- name[name_exists][[1]]
+    value <- config[[name_primary]]
+  }
+
+  if (is.function(value)) value <- value()
+  value
+}
+
+spark_config_integer <- function(config, name, default = NULL) {
+  as.integer(spark_config_value(config, name, default))
+}
+
+spark_config_logical <- function(config, name, default = NULL) {
+  as.logical(spark_config_value(config, name, default))
 }
 #' Check whether the connection is open
 #'
@@ -23,6 +53,62 @@ spark_config_value <- function(config, name, default = NULL) {
 connection_is_open <- function(sc) {
   UseMethod("connection_is_open")
 }
+read_bin <- function(con, what, n, endian = NULL) {
+  UseMethod("read_bin")
+}
+
+read_bin.default <- function(con, what, n, endian = NULL) {
+  if (is.null(endian)) readBin(con, what, n) else readBin(con, what, n, endian = endian)
+}
+
+read_bin_wait <- function(con, what, n, endian = NULL) {
+  sc <- con
+  con <- if (!is.null(sc$state) && identical(sc$state$use_monitoring, TRUE)) sc$monitoring else sc$backend
+
+  timeout <- spark_config_value(sc$config, "sparklyr.backend.timeout", 30 * 24 * 60 * 60)
+  progressInterval <- spark_config_value(sc$config, "sparklyr.progress.interval", 3)
+
+  result <- if (is.null(endian)) readBin(con, what, n) else readBin(con, what, n, endian = endian)
+
+  progressTimeout <- Sys.time() + progressInterval
+  if (is.null(sc$state$progress))
+    sc$state$progress <- new.env()
+  progressUpdated <- FALSE
+
+  waitInterval <- 0
+  commandStart <- Sys.time()
+  while(length(result) == 0 && commandStart + timeout > Sys.time()) {
+    Sys.sleep(waitInterval)
+    waitInterval <- min(0.1, waitInterval + 0.01)
+
+    result <- if (is.null(endian)) readBin(con, what, n) else readBin(con, what, n, endian = endian)
+
+    if (Sys.time() > progressTimeout) {
+      progressTimeout <- Sys.time() + progressInterval
+      if (exists("connection_progress")) {
+        connection_progress(sc)
+        progressUpdated <- TRUE
+      }
+    }
+  }
+
+  if (progressUpdated) connection_progress_terminated(sc)
+
+  if (commandStart + timeout <= Sys.time()) {
+    stop("Operation timed out, increase config option sparklyr.backend.timeout if needed.")
+  }
+
+  result
+}
+
+read_bin.spark_connection <- function(con, what, n, endian = NULL) {
+  read_bin_wait(con, what, n, endian)
+}
+
+read_bin.spark_worker_connection <- function(con, what, n, endian = NULL) {
+  read_bin_wait(con, what, n, endian)
+}
+
 readObject <- function(con) {
   # Read type first
   type <- readType(con)
@@ -49,8 +135,13 @@ readTypedObject <- function(con, type) {
 
 readString <- function(con) {
   stringLen <- readInt(con)
-  raw <- readBin(con, raw(), stringLen, endian = "big")
-  string <- rawToChar(raw)
+  string <- ""
+
+  if (stringLen > 0) {
+    raw <- read_bin(con, raw(), stringLen, endian = "big")
+    string <- rawToChar(raw)
+  }
+
   Encoding(string) <- "UTF-8"
   string
 }
@@ -61,19 +152,28 @@ readDateArray <- function(con, n = 1) {
 }
 
 readInt <- function(con, n = 1) {
-  readBin(con, integer(), n = n, endian = "big")
+  if (n == 0)
+    integer(0)
+  else
+    read_bin(con, integer(), n = n, endian = "big")
 }
 
 readDouble <- function(con, n = 1) {
-  readBin(con, double(), n = n, endian = "big")
+  if (n == 0)
+    double(0)
+  else
+    read_bin(con, double(), n = n, endian = "big")
 }
 
 readBoolean <- function(con, n = 1) {
-  as.logical(readInt(con, n = n))
+  if (n == 0)
+    logical(0)
+  else
+    as.logical(readInt(con, n = n))
 }
 
 readType <- function(con) {
-  rawToChar(readBin(con, "raw", n = 1L))
+  rawToChar(read_bin(con, "raw", n = 1L))
 }
 
 readDate <- function(con) {
@@ -81,13 +181,19 @@ readDate <- function(con) {
 }
 
 readTime <- function(con, n = 1) {
-  t <- readDouble(con, n)
-  timeNA <- as.POSIXct(0, origin = "1970-01-01", tz = "UTC")
+  if (identical(n, 0))
+    as.POSIXct(character(0))
+  else {
+    t <- readDouble(con, n)
+    timeNA <- as.POSIXct(0, origin = "1970-01-01", tz = "UTC")
 
-  r <- as.POSIXct(t, origin = "1970-01-01", tz = "UTC")
-  if (getOption("sparklyr.collect.datechars", FALSE)) as.character(r) else {
-    r[r == timeNA] <- as.POSIXct(NA)
-    r
+    r <- as.POSIXct(t, origin = "1970-01-01", tz = "UTC")
+    if (getOption("sparklyr.collect.datechars", FALSE))
+      as.character(r)
+    else {
+      r[r == timeNA] <- as.POSIXct(NA)
+      r
+    }
   }
 }
 
@@ -170,13 +276,16 @@ readStruct <- function(con) {
 
 readRaw <- function(con) {
   dataLen <- readInt(con)
-  readBin(con, raw(), as.integer(dataLen), endian = "big")
+  if (dataLen == 0)
+    raw()
+  else
+    read_bin(con, raw(), as.integer(dataLen), endian = "big")
 }
 wait_connect_gateway <- function(gatewayAddress, gatewayPort, config, isStarting) {
   waitSeconds <- if (isStarting)
-    spark_config_value(config, "sparklyr.gateway.start.timeout", 60)
+    spark_config_value(config, "sparklyr.connect.timeout", 60)
   else
-    spark_config_value(config, "sparklyr.gateway.connect.timeout", 1)
+    spark_config_value(config, "sparklyr.gateway.timeout", 1)
 
   gateway <- NULL
   commandStart <- Sys.time()
@@ -184,7 +293,7 @@ wait_connect_gateway <- function(gatewayAddress, gatewayPort, config, isStarting
   while (is.null(gateway) && Sys.time() < commandStart + waitSeconds) {
     tryCatch({
       suppressWarnings({
-        timeout <- spark_config_value(config, "sparklyr.monitor.timeout", 1)
+        timeout <- spark_config_value(config, "sparklyr.gateway.interval", 1)
         gateway <- socketConnection(host = gatewayAddress,
                                     port = gatewayPort,
                                     server = FALSE,
@@ -195,7 +304,7 @@ wait_connect_gateway <- function(gatewayAddress, gatewayPort, config, isStarting
     }, error = function(err) {
     })
 
-    startWait <- spark_config_value(config, "sparklyr.gateway.start.wait", 50 / 1000)
+    startWait <- spark_config_value(config, "sparklyr.gateway.wait", 50 / 1000)
     Sys.sleep(startWait)
   }
 
@@ -211,9 +320,9 @@ spark_gateway_commands <- function() {
 
 query_gateway_for_port <- function(gateway, sessionId, config, isStarting) {
   waitSeconds <- if (isStarting)
-    spark_config_value(config, "sparklyr.gateway.start.timeout", 60)
+    spark_config_value(config, "sparklyr.connect.timeout", 60)
   else
-    spark_config_value(config, "sparklyr.gateway.connect.timeout", 1)
+    spark_config_value(config, "sparklyr.gateway.timeout", 1)
 
   writeInt(gateway, spark_gateway_commands()[["GetPorts"]])
   writeInt(gateway, sessionId)
@@ -258,7 +367,7 @@ spark_connect_gateway <- function(
   if (is.null(gateway)) {
     if (isStarting)
       stop(
-        "Gateway in port (", gatewayPort, ") did not respond.")
+        "Gateway in ", gatewayAddress, ":", gatewayPort, " did not respond.")
 
     NULL
   }
@@ -280,7 +389,7 @@ spark_connect_gateway <- function(
       close(gateway)
 
       if (isStarting)
-        stop("Gateway in port (", gatewayPort, ") does not have the requested session registered")
+        stop("Gateway in ", gatewayAddress, ":", gatewayPort, " does not have the requested session registered")
 
       NULL
     } else if(redirectGatewayPort != gatewayPort) {
@@ -296,22 +405,55 @@ spark_connect_gateway <- function(
     }
   }
 }
-core_invoke_method <- function(sc, static, object, method, ...)
+core_invoke_sync_socket <- function(sc)
 {
-  if (is.null(sc)) {
-    stop("The connection is no longer valid.")
+  flush <- c(1)
+  while(length(flush) > 0) {
+    flush <- readBin(sc$backend, raw(), 1000)
+
+    # while flushing monitored connections we don't want to hang forever
+    if (identical(sc$state$use_monitoring, TRUE)) break;
   }
+}
 
-  # if the object is a jobj then get it's id
-  if (inherits(object, "spark_jobj"))
-    object <- object$id
+core_invoke_sync <- function(sc)
+{
+  # sleep until connection clears is back on valid state
+  while (!core_invoke_synced(sc)) {
+    Sys.sleep(1)
+    core_invoke_sync_socket(sc)
+  }
+}
 
+core_invoke_cancel_running <- function(sc)
+{
+  if (is.null(spark_context(sc)))
+    return()
+
+  # if something fails while using a monitored connection we don't cancel jobs
+  if (identical(sc$state$use_monitoring, TRUE))
+    return()
+
+  # if something fails while cancelling jobs we don't cancel jobs, this can
+  # happen in OutOfMemory errors that shut down the spark context
+  if (identical(sc$state$cancelling_all_jobs, TRUE))
+    return()
+
+  connection_progress_context(sc, function() {
+    sc$state$cancelling_all_jobs <- TRUE
+    on.exit(sc$state$cancelling_all_jobs <- FALSE)
+    invoke(spark_context(sc), "cancelAllJobs")
+  })
+
+  if (exists("connection_progress_terminated")) connection_progress_terminated(sc)
+}
+
+write_bin_args <- function(backend, object, static, method, args) {
   rc <- rawConnection(raw(), "r+")
   writeString(rc, object)
   writeBoolean(rc, static)
   writeString(rc, method)
 
-  args <- list(...)
   writeInt(rc, length(args))
   writeArgs(rc, args)
   bytes <- rawConnectionValue(rc)
@@ -323,8 +465,75 @@ core_invoke_method <- function(sc, static, object, method, ...)
   con <- rawConnectionValue(rc)
   close(rc)
 
-  backend <- sc$backend
   writeBin(con, backend)
+}
+
+core_invoke_synced <- function(sc)
+{
+  if (is.null(sc))
+    stop("The connection is no longer valid.")
+
+  backend <- core_invoke_socket(sc)
+  echo_id <- "sparklyr"
+
+  write_bin_args(backend, "Handler", TRUE, "echo", echo_id)
+
+  returnStatus <- readInt(backend)
+
+  if (length(returnStatus) == 0 || returnStatus != 0) {
+    FALSE
+  }
+  else {
+    object <- readObject(sc)
+    identical(object, echo_id)
+  }
+}
+
+core_invoke_socket <- function(sc) {
+  if (identical(sc$state$use_monitoring, TRUE))
+    sc$monitoring
+  else
+    sc$backend
+}
+
+core_invoke_socket_name <- function(sc) {
+  if (identical(sc$state$use_monitoring, TRUE))
+    "monitoring"
+  else
+    "backend"
+}
+
+core_invoke_method <- function(sc, static, object, method, ...)
+{
+  if (is.null(sc))
+    stop("The connection is no longer valid.")
+
+  args <- list(...)
+
+  # initialize status if needed
+  if (is.null(sc$state$status))
+    sc$state$status <- list()
+
+  # choose connection socket
+  backend <- core_invoke_socket(sc)
+  connection_name <- core_invoke_socket_name(sc)
+
+  if (!identical(object, "Handler")) {
+    # if connection still running, sync to valid state
+    if (identical(sc$state$status[[connection_name]], "running"))
+      core_invoke_sync(sc)
+
+    # while exiting this function, if interrupted (still running), cancel server job
+    on.exit(core_invoke_cancel_running(sc))
+
+    sc$state$status[[connection_name]] <- "running"
+  }
+
+  # if the object is a jobj then get it's id
+  if (inherits(object, "spark_jobj"))
+    object <- object$id
+
+  write_bin_args(backend, object, static, method, args)
 
   if (identical(object, "Handler") &&
       (identical(method, "terminateBackend") || identical(method, "stopBackend"))) {
@@ -332,18 +541,17 @@ core_invoke_method <- function(sc, static, object, method, ...)
     return(NULL)
   }
 
-  returnStatus <- readInt(backend)
+  returnStatus <- readInt(sc)
 
   if (length(returnStatus) == 0) {
     # read the spark log
     msg <- core_read_spark_log_error(sc)
-    close(sc$backend)
-    close(sc$monitor)
+
     withr::with_options(list(
       warning.length = 8000
     ), {
       stop(
-        "Unexpected state in sparklyr backend, terminating connection: ",
+        "Unexpected state in sparklyr backend: ",
         msg,
         call. = FALSE)
     })
@@ -351,7 +559,7 @@ core_invoke_method <- function(sc, static, object, method, ...)
 
   if (returnStatus != 0) {
     # get error message from backend and report to R
-    msg <- readString(backend)
+    msg <- readString(sc)
     withr::with_options(list(
       warning.length = 8000
     ), {
@@ -367,9 +575,11 @@ core_invoke_method <- function(sc, static, object, method, ...)
     })
   }
 
-  class(backend) <- c(class(backend), "shell_backend")
+  object <- readObject(sc)
+
+  sc$state$status[[connection_name]] <- "ready"
+  on.exit(NULL)
 
-  object <- readObject(backend)
   attach_connection(object, sc)
 }
 
@@ -377,6 +587,14 @@ jobj_subclass.shell_backend <- function(con) {
   "shell_jobj"
 }
 
+jobj_subclass.spark_connection <- function(con) {
+  "shell_jobj"
+}
+
+jobj_subclass.spark_worker_connection <- function(con) {
+  "shell_jobj"
+}
+
 core_handle_known_errors <- function(sc, msg) {
   # Some systems might have an invalid hostname that Spark <= 2.0.1 fails to handle
   # gracefully and triggers unexpected errors such as #532. Under these versions,
@@ -430,6 +648,9 @@ spark_jobj <- function(x, ...) {
   UseMethod("spark_jobj")
 }
 
+spark_jobj_id <- function(x) {
+  x$id
+}
 
 #' @export
 spark_jobj.default <- function(x, ...) {
@@ -695,25 +916,6 @@ writeBoolean <- function(con, value) {
   writeInt(con, as.integer(value))
 }
 
-writeRawSerialize <- function(outputCon, batch) {
-  outputSer <- serialize(batch, ascii = FALSE, connection = NULL)
-  writeRaw(outputCon, outputSer)
-}
-
-writeRowSerialize <- function(outputCon, rows) {
-  invisible(lapply(rows, function(r) {
-    bytes <- serializeRow(r)
-    writeRaw(outputCon, bytes)
-  }))
-}
-
-serializeRow <- function(row) {
-  rawObj <- rawConnection(raw(0), "wb")
-  on.exit(close(rawObj))
-  writeList(rawObj, row)
-  rawConnectionValue(rawObj)
-}
-
 writeRaw <- function(con, batch) {
   writeInt(con, length(batch))
   writeBin(batch, con, endian = "big")
@@ -817,6 +1019,7 @@ worker_config_serialize <- function(config) {
     spark_config_value(config, "sparklyr.worker.gateway.port", "8880"),
     spark_config_value(config, "sparklyr.worker.gateway.address", "localhost"),
     if (isTRUE(config$profile)) "TRUE" else "FALSE",
+    if (isTRUE(config$schema)) "TRUE" else "FALSE",
     sep = ";"
   )
 }
@@ -828,10 +1031,11 @@ worker_config_deserialize <- function(raw) {
     debug = as.logical(parts[[1]]),
     sparklyr.gateway.port = as.integer(parts[[2]]),
     sparklyr.gateway.address = parts[[3]],
-    profile = as.logical(parts[[4]])
+    profile = as.logical(parts[[4]]),
+    schema = as.logical(parts[[5]])
   )
 }
-spark_worker_apply <- function(sc) {
+spark_worker_apply <- function(sc, config) {
   hostContextId <- worker_invoke_method(sc, FALSE, "Handler", "getHostContext")
   worker_log("retrieved worker context id ", hostContextId)
 
@@ -868,6 +1072,11 @@ spark_worker_apply <- function(sc) {
     .libPaths(unbundlePath)
     worker_log("updated .libPaths with bundle packages")
   }
+  else {
+    spark_env <- worker_invoke_static(sc, "org.apache.spark.SparkEnv", "get")
+    spark_libpaths <- worker_invoke(worker_invoke(spark_env, "conf"), "get", "spark.r.libpaths", NULL)
+    if (!is.null(spark_libpaths)) .libPaths(spark_libpaths)
+  }
 
   grouped_by <- worker_invoke(context, "getGroupBy")
   grouped <- !is.null(grouped_by) && length(grouped_by) > 0
@@ -895,6 +1104,10 @@ spark_worker_apply <- function(sc) {
     }
   }
 
+  if (identical(config$schema, TRUE)) {
+    worker_log("is running to compute schema")
+  }
+
   columnNames <- worker_invoke(context, "getColumns")
 
   if (!grouped) groups <- list(list(groups))
@@ -908,15 +1121,25 @@ spark_worker_apply <- function(sc) {
     df <- do.call(rbind.data.frame, c(data, list(stringsAsFactors = FALSE)))
 
     # rbind removes Date classes so we re-assign them here
-    if (length(data) > 0 && ncol(df) > 0 && nrow(df) > 0 &&
-        any(sapply(data[[1]], function(e) class(e)[[1]]) %in% c("Date", "POSIXct"))) {
-      first_row <- data[[1]]
-      for (idx in seq_along(first_row)) {
-        first_class <- class(first_row[[idx]])[[1]]
-        if (identical(first_class, "Date")) {
-          df[[idx]] <- as.Date(df[[idx]], origin = "1970-01-01")
-        } else if (identical(first_class, "POSIXct")) {
-          df[[idx]] <- as.POSIXct(df[[idx]], origin = "1970-01-01")
+    if (length(data) > 0 && ncol(df) > 0 && nrow(df) > 0) {
+
+      if (any(sapply(data[[1]], function(e) class(e)[[1]]) %in% c("Date", "POSIXct"))) {
+        first_row <- data[[1]]
+        for (idx in seq_along(first_row)) {
+          first_class <- class(first_row[[idx]])[[1]]
+          if (identical(first_class, "Date")) {
+            df[[idx]] <- as.Date(df[[idx]], origin = "1970-01-01")
+          } else if (identical(first_class, "POSIXct")) {
+            df[[idx]] <- as.POSIXct(df[[idx]], origin = "1970-01-01")
+          }
+        }
+      }
+
+      # cast column to correct type, for instance, when dealing with NAs.
+      for (i in 1:ncol(df)) {
+        target_type <- funcContext$column_types[[i]]
+        if (!is.null(target_type) && class(df[[i]]) != target_type) {
+        df[[i]] <- do.call(paste("as", target_type, sep = "."), args = list(df[[i]]))
         }
       }
     }
@@ -932,7 +1155,7 @@ spark_worker_apply <- function(sc) {
       closure_params <- length(formals(closure))
       closure_args <- c(
         list(df),
-        if (!is.null(funcContext)) list(funcContext) else NULL,
+        if (!is.null(funcContext$user_context)) list(funcContext$user_context) else NULL,
         as.list(
           if (nrow(df) > 0)
             lapply(grouped_by, function(group_by_name) df[[group_by_name]][[1]])
@@ -945,19 +1168,37 @@ spark_worker_apply <- function(sc) {
       result <- do.call(closure, closure_args)
       worker_log("computed closure")
 
-      if (!identical(class(result), "data.frame")) {
+      if (!"data.frame" %in% class(result)) {
         worker_log("data.frame expected but ", class(result), " found")
-        result <- data.frame(result)
+        result <- as.data.frame(result)
       }
 
       if (!is.data.frame(result)) stop("Result from closure is not a data.frame")
     }
 
     if (grouped) {
-      new_column_values <- lapply(grouped_by, function(grouped_by_name) df[[grouped_by_name]][[1]])
-      names(new_column_values) <- grouped_by
+      if (nrow(result) > 0) {
+        new_column_values <- lapply(grouped_by, function(grouped_by_name) df[[grouped_by_name]][[1]])
+        names(new_column_values) <- grouped_by
 
-      result <- do.call("cbind", list(new_column_values, result))
+        if("AsIs" %in% class(result)) class(result) <- class(result)[-match("AsIs", class(result))]
+        result <- do.call("cbind", list(new_column_values, result))
+
+        names(result) <- gsub("\\.", "_", make.unique(names(result)))
+      }
+      else {
+        result <- NULL
+      }
+    }
+
+    firstClass <- function(e) class(e)[[1]]
+
+    if (identical(config$schema, TRUE)) {
+      worker_log("updating schema")
+      result <- data.frame(
+        names = paste(names(result), collapse = "|"),
+        types = paste(lapply(result, firstClass), collapse = "|")
+      )
     }
 
     all_results <- rbind(all_results, result)
@@ -965,6 +1206,7 @@ spark_worker_apply <- function(sc) {
 
   if (!is.null(all_results) && nrow(all_results) > 0) {
     worker_log("updating ", nrow(all_results), " rows")
+
     all_data <- lapply(1:nrow(all_results), function(i) as.list(all_results[i,]))
 
     worker_invoke(context, "setResultArraySeq", all_data)
@@ -1041,14 +1283,16 @@ spark_worker_connect <- function(
   worker_log("is connecting to backend session")
 
   tryCatch({
-    # set timeout for socket connection
-    timeout <- spark_config_value(config, "sparklyr.backend.timeout", 30 * 24 * 60 * 60)
+    interval <- spark_config_value(config, "sparklyr.backend.interval", 1)
+
     backend <- socketConnection(host = "localhost",
                                 port = gatewayInfo$backendPort,
                                 server = FALSE,
-                                blocking = TRUE,
+                                blocking = interval > 0,
                                 open = "wb",
-                                timeout = timeout)
+                                timeout = interval)
+
+    class(backend) <- c(class(backend), "shell_backend")
   }, error = function(err) {
     close(gatewayInfo$gateway)
 
@@ -1068,7 +1312,7 @@ spark_worker_connect <- function(
     # spark_shell_connection
     spark_home = NULL,
     backend = backend,
-    monitor = gatewayInfo$gateway,
+    gateway = gatewayInfo$gateway,
     output_file = NULL
   ))
 
@@ -1081,7 +1325,7 @@ connection_is_open.spark_worker_connection <- function(sc) {
   bothOpen <- FALSE
   if (!identical(sc, NULL)) {
     tryCatch({
-      bothOpen <- isOpen(sc$backend) && isOpen(sc$monitor)
+      bothOpen <- isOpen(sc$backend) && isOpen(sc$gateway)
     }, error = function(e) {
     })
   }
@@ -1190,7 +1434,7 @@ spark_worker_main <- function(
     sc <- spark_worker_connect(sessionId, backendPort, config)
     worker_log("is connected")
 
-    spark_worker_apply(sc)
+    spark_worker_apply(sc, config)
 
     if (identical(config$profile, TRUE)) {
       # utils::Rprof(NULL)
